<?php
header('Content-Type: application/json');
require 'config.php'; // Einbinden der Konfigurationsdatei

function logline($text, $wert = "") {
    $logFile = 'logfile.log';
    $logLine = "";

    if ($text == "<H>") {
        $logLine .= "┌────────────────────────────────────────────────────────────────────────┐\n";
        $logLine .= "│ " . str_pad($wert, 71) . " │\n";
        $logLine .= "└────────────────────────────────────────────────────────────────────────┘\n";
    } else {
        $logLine .= str_pad($text, 50) . ": " . $wert . "\n";
    }

    file_put_contents($logFile, $logLine, FILE_APPEND);
}

function openDbf($filename) {
    $dbf = fopen($filename, 'rb+');
    if (!$dbf) {
        logline("Error", "Kann die DBF-Datei nicht öffnen.");
        throw new Exception("Kann die DBF-Datei nicht öffnen.");
    }
    return $dbf;
}

function getDbfHeader($dbf) {
    $header = fread($dbf, 32);
    $data = unpack('VrecordCount/vheaderLength/vrecordLength', substr($header, 4, 8));
    return $data;
}

function padValue($value, $length, $type) {
    switch ($type) {
        case 'C':
            return str_pad($value, $length);
        case 'L':
            return $value ? 'T' : 'F';
        default:
            return str_pad($value, $length);
    }
}

function addRecordToDbf($filename, $record) {
    $dbf = openDbf($filename);
    $header = getDbfHeader($dbf);

    $recordCount = $header['recordCount'];
    $newRecordNumber = $recordCount + 1;

    $headerLength = $header['headerLength'];
    $recordLength = $header['recordLength'];

    fseek($dbf, $headerLength + $recordCount * $recordLength);

    $fields = [
        ['name' => 'FIRST', 'type' => 'C', 'length' => 20],
        ['name' => 'LAST', 'type' => 'C', 'length' => 20],
        ['name' => 'STREET', 'type' => 'C', 'length' => 30],
        ['name' => 'CITY', 'type' => 'C', 'length' => 30],
        ['name' => 'STATE', 'type' => 'C', 'length' => 2],
        ['name' => 'MARRIED', 'type' => 'L', 'length' => 1]
    ];

    $recordData = ' ';
    foreach ($fields as $field) {
        $value = isset($record[$field['name']]) ? $record[$field['name']] : '';
        $recordData .= padValue($value, $field['length'], $field['type']);
    }

    fwrite($dbf, $recordData);

    fseek($dbf, 4);
    fwrite($dbf, pack('V', $newRecordNumber));

    fclose($dbf);

    logline("Neuer Datensatz hinzugefügt", json_encode($record));

    echo json_encode(["success" => true, "message" => "Record added successfully"]);
}

$data = json_decode(file_get_contents('php://input'), true);
if ($data) {
    addRecordToDbf(DBF_FILE_PATH, [
        'FIRST' => $data['first'],
        'LAST' => $data['last'],
        'STREET' => $data['street'],
        'CITY' => $data['city'],
        'STATE' => $data['state'],
        'MARRIED' => isset($data['married']) ? $data['married'] : false
    ]);
} else {
    echo json_encode(["success" => false, "message" => "Invalid input"]);
}
?>
