<?php
header('Content-Type: application/json');
require 'config.php'; // Einbinden der Konfigurationsdatei

function logline($text, $wert = "") {
    $logFile = 'logfile.log';
    $logLine = "";

    if ($text == "<H>") {
        $logLine .= "┌────────────────────────────────────────────────────────────────────────┐\n";
        $logLine .= "│ " . str_pad($wert, 71) . " │\n";
        $logLine .= "└────────────────────────────────────────────────────────────────────────┘\n";
    } else {
        $logLine .= str_pad($text, 50) . ": " . $wert . "\n";
    }

//    file_put_contents($logFile, $logLine, FILE_APPEND);
}

function convertToASCII($str) {
    return preg_replace('/[^a-zA-Z0-9]/', 'x', $str);
}

function read_memo($memo_file, $block_num) {
    static $memo = null;
    static $memo_file_path = null;

    if ($memo_file_path !== $memo_file) {
        if ($memo !== null) {
            fclose($memo);
        }

        $memo_file_path = $memo_file;
        $memo_path = pathinfo($memo_file, PATHINFO_DIRNAME) . '/' . pathinfo($memo_file, PATHINFO_FILENAME) . '.fpt';
        
        logline("Memo-Dateipfad", $memo_path);

        if (!file_exists($memo_path)) {
            logline("Error", "Memo-Datei nicht gefunden.");
            return "";
        }

        $memo = fopen($memo_path, 'rb');
        if (!$memo) {
            logline("Error", "Konnte die Memo-Datei nicht öffnen.");
            return "";
        }
    }

    fseek($memo, $block_num * 512);
    $block_header = fread($memo, 8);
    if (strlen($block_header) < 8) {
        logline("Error", "Ungültiger Memo-Block-Header. Block-Nummer: " . $block_num);
        logline("Memo-Blocknummer (Hex)", dechex($block_num));
        return ""; // Oder einen spezifischen Fehlerwert zurückgeben
    }

    try {
        $memo_length = unpack('N', substr($block_header, 4, 4))[1];
        if ($memo_length <= 0 || $memo_length > 512) {
            throw new Exception("Ungültige Memo-Länge. Block-Nummer: " . $block_num . ", Länge: " . $memo_length);
        }

        $memo_data = fread($memo, $memo_length);
        if (strlen($memo_data) < $memo_length) {
            throw new Exception("Ungültige Memo-Daten. Block-Nummer: " . $block_num . ", Gelesene Länge: " . strlen($memo_data) . ", Erwartete Länge: " . $memo_length);
        }

        return convertToASCII($memo_data);

    } catch (Exception $e) {
        logline("Error", $e->getMessage());
        logline("Memo-Blocknummer (Hex)", dechex($block_num));
        return ""; // Oder einen spezifischen Fehlerwert zurückgeben
    }
}

function read_dbf($file_path, $start = null, $end = null, $batch_size = 1000, $read_memo_fields = true) {
    $file = fopen($file_path, 'rb');
    if (!$file) {
        logline("Error", "Konnte die Datei nicht öffnen.");
        die(json_encode(['error' => "Konnte die Datei nicht öffnen."]));
    }

    // Header lesen
    $header = fread($file, 32);
    $num_records = unpack('V', substr($header, 4, 4))[1];
    $header_size = unpack('v', substr($header, 8, 2))[1];
    $record_size = unpack('v', substr($header, 10, 2))[1];

    // Felddeskriptoren lesen
    $field_descriptors = [];
    fseek($file, 32);
    while (true) {
        $field_descriptor = fread($file, 32);
        if (ord($field_descriptor[0]) == 0x0D) { // Ende der Felddeskriptoren
            break;
        }
        $field_name = rtrim(substr($field_descriptor, 0, 11));
        $field_type = $field_descriptor[11];
        $field_length = ord($field_descriptor[16]);
        $field_descriptors[] = [
            'name' => $field_name,
            'type' => $field_type,
            'length' => $field_length
        ];
    }

    logline("Header gelesen", json_encode($header));
    logline("Felddeskriptoren", json_encode($field_descriptors));

    // Datensätze lesen
    $start = $start !== null ? max($start, 1) : 1;
    $end = $end !== null ? min($end, $num_records) : $num_records;

    logline("Lese Datensätze von", "$start bis $end");

    $records = [];
    for ($batch_start = $start; $batch_start <= $end; $batch_start += $batch_size) {
        $batch_end = min($batch_start + $batch_size - 1, $end);

        fseek($file, $header_size + ($batch_start - 1) * $record_size);
        for ($i = $batch_start; $i <= $batch_end; $i++) {
            $record = fread($file, $record_size);
            if (strlen($record) < $record_size) {
                logline("Error", "Ungültige Datensatzlänge.");
                continue;
            }

            if (ord($record[0]) != 0x2A) { // Nur nicht gelöschte Datensätze lesen
                $record_data = ['recno' => $i]; // recno hinzufügen
                $offset = 1;
                foreach ($field_descriptors as $field) {
                    $field_value = rtrim(substr($record, $offset, $field['length']));
                    if ($read_memo_fields && $field['type'] == 'M') { // Memo-Feld
                        $block_num = unpack('V', str_pad($field_value, 4, "\0", STR_PAD_LEFT))[1];
                        logline("Memo-Blocknummer (Dezimal)", $block_num);
                        logline("Memo-Blocknummer (Hex)", dechex($block_num));
                        $field_value = read_memo($file_path, $block_num);
                    }
                    $record_data[$field['name']] = convertToASCII($field_value);
                    $offset += $field['length'];
                }
                $records[] = $record_data;
            }
        }
    }

    fclose($file);

    return ['totalRecords' => $num_records, 'records' => $records];
}

// Pfad zur DBF-Datei angeben
$input = json_decode(file_get_contents('php://input'), true);
$start = isset($input['start']) && $input['start'] !== '' ? intval($input['start']) : null;
$end = isset($input['end']) && $input['end'] !== '' ? intval($input['end']) : null;
$filterServer = isset($input['filterServer']) ? filter_var($input['filterServer'], FILTER_VALIDATE_BOOLEAN) : false;
$read_memo_fields = isset($input['memoFields']) ? filter_var($input['memoFields'], FILTER_VALIDATE_BOOLEAN) : true;

// Debugging-Informationen
logline("<H>", time());
logline("Empfangene POST-Daten", json_encode($input));
logline("Start", json_encode($start));
logline("End", json_encode($end));
logline("Filter Server", json_encode($filterServer));
logline("Memo-Felder einlesen", json_encode($read_memo_fields));

$batch_size = 1000; // Set the batch size here

$records = $filterServer ? read_dbf(DBF_FILE_PATH, $start, $end, $batch_size, $read_memo_fields) : read_dbf(DBF_FILE_PATH, null, null, $batch_size, $read_memo_fields);

// Gebe das Array als JSON zurück
logline("Rückgabedaten", json_encode($records, JSON_UNESCAPED_UNICODE));
echo json_encode($records, JSON_UNESCAPED_UNICODE);
?>
